import setuptools


setuptools.setup(
    name='xy-libcollection',
    version='0.0.7',
    author='xueyan',
    author_email='xueyan0407@outlook.com',
    description="xueyan's collection libs",
    long_description='#A long desc',
    long_descripthin_content_type='text/markdown',
    url='https://github.com/qimiaoxue/libcollection',
    packages=setuptools.find_packages(),
    classifiers=[],
)
